/**
 * Highly inspired from Dailymotion JS SDK available at https://github.com/dailymotion/dailymotion-sdk-js
 *
 * Copyright Work Bandits SAS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 *
 * Contains the public method ``DI.api`` and the internal implementation
 * ``DI.ApiServer``.
 *
 * @provides di.api
 * @requires di.prelude
 *           di.qs
 *           di.json
 *           di.array
 */

/**
 * API calls.
 *
 * @class DI
 * @static
 * @access private
 */
DI.provide('', {
    api: function() {
        DI.ApiServer.call.apply(DI.ApiServer, arguments);
    }
});

/**
 * API call implementations.
 *
 * @class DI.ApiServer
 * @access private
 */
DI.provide('ApiServer', {
    METHODS: ['get', 'post'],
    endpoint: DI._domain['endpoint'],
    _callbacks: {},

    /**
     * Make a API call to Dingg.it RESTful API.
     *
     * Except the path, all arguments to this function are optional. So any of
     * these are valid:
     *
     *   DI.api('/me') // throw away the response
     *   DI.api('/me', function(r) { console.log(r) })
     *   DI.api('/me', { fields: 'email' }); // throw away response
     *   DI.api('/me', { fields: 'email' }, function(r) { console.log(r) });
     *   DI.api('/12345678', 'delete', function(r) { console.log(r) });
     *   DI.api(
     *     '/me/feed',
     *     'post',
     *     { body: 'hi there' },
     *     function(r) { console.log(r) }
     *   );
     *
     * @access private
     * @param path   {String}   the url path
     * @param method {String}   the http method
     * @param params {Object}   the parameters for the query
     * @param cb     {Function} the callback function to handle the response
     */
    call: function() {
        var args = Array.prototype.slice.call(arguments),
            path = args.shift(),
            next = args.shift(),
            method,
            params,
            cb;

        while (next) {
            var type = typeof next;
            if (type === 'string' && !method) {
                method = next.toLowerCase();
            } else if (type === 'function' && !cb) {
                cb = next;
            } else if (type === 'object' && !params) {
                params = next;
            } else {
                DI.log('Invalid argument passed to DI.api(): ' + next);
                return;
            }
            next = args.shift();
        }

        method = method || 'get';
        params = params || {};

        // remove prefix slash if one is given, as it's already in the base url
        if (path[0] === '/') {
            path = path.substr(1);
        }

        if (DI.Array.indexOf(DI.ApiServer.METHODS, method) < 0) {
            DI.log('Invalid method passed to DI.api(): ' + method);
            return;
        }

        DI.ApiServer.oauthRequest(path, method, params, cb);
    },

    /**
     * Add the oauth parameter, and fire off a request.
     *
     * @access private
     * @param path   {String}   the request path
     * @param method {String}   the http method
     * @param params {Object}   the parameters for the query
     * @param cb     {Function} the callback function to handle the response
     */
    oauthRequest: function(path, method, params, cb) {
        // add oauth token if we have one
        if (DI.getSession) {
            var session = DI.getSession();
            if (session && session.access_token && !params.access_token) {
                params.access_token = session.access_token;
            }
        }

        DI.ApiServer.jsonp(path, method, DI.JSON.flatten(params), cb);
    },

    /**
     * Basic JSONP Support.
     *
     * @access private
     * @param path   {String}   the request path
     * @param method {String}   the http method
     * @param params {Object}   the parameters for the query
     * @param cb     {Function} the callback function to handle the response
     */
    jsonp: function(path, method, params, cb) {
        var g = DI.guid(),
            script = document.createElement('script');

        // jsonp needs method overrides as the request itself is always a GET
        params.method = method;
        params.callback = 'DI.ApiServer._callbacks.' + g;

        var url = (DI.ApiServer.endpoint + path + (path.indexOf('?') > -1 ? '&' : '?') + DI.QS.encode(params));

	DI.log('URL called: ' + url + ' URL length: ' + url.length);

        if (url.length > 2000) {
            throw new Error('JSONP only support a maximum of 2000 bytes of input.');
        }

        // this is the JSONP callback invoked by the response
        DI.ApiServer._callbacks[g] = function(response) {
            cb && cb(response);
            delete DI.ApiServer._callbacks[g];
            script.src = null;
            script.parentNode.removeChild(script);
        };

        script.src = url;
        document.getElementsByTagName('head')[0].appendChild(script);
    }
});
