/**
 * Highly inspired from Dailymotion JS SDK available at https://github.com/dailymotion/dailymotion-sdk-js
 *
 * Copyright Work Bandits SAS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 *
 * @provides di.auth
 * @requires di.prelude
 *           di.qs
 *           di.event
 *           di.json
 *           di.api
 */

/**
 * Authentication, Authorization & Sessions.
 *
 * @class di
 * @static
 * @access private
 */
DI.provide('', {
    getLoginStatus: function(cb) {
        if (cb) {
            cb({status: DI._userStatus, session: DI._session});
        }
    },

    getSession: function() {
        if (DI._session && 'expires' in DI._session && new Date().getTime() > DI._session.expires * 1000) {
            DI.Auth.setSession(null, 'notConnected');
        }

        return DI._session;
    },

    login: function(cb, opts) {
        // we try to place it at the center of the current window
        var screenX = typeof window.screenX != 'undefined' ? window.screenX : window.screenLeft,
            screenY = typeof window.screenY != 'undefined' ? window.screenY : window.screenTop,
            outerWidth = typeof window.outerWidth != 'undefined' ? window.outerWidth : document.documentElement.clientWidth,
            outerHeight = typeof window.outerHeight != 'undefined' ? window.outerHeight : (document.documentElement.clientHeight - 22), // 22 = IE toolbar height
            width = 600,
            height = 420,
            left = parseInt(screenX + ((outerWidth - width) / 2), 10),
            top = parseInt(screenY + ((outerHeight - height) / 2.5), 10),
            features = 'width=' + width + ',height=' + height + ',left=' + left + ',top=' + top;

		var redirect_uri = document.location.href.replace('#_=_', '');

        opts = DI.copy(opts || {}, {
            client_id: DI._apiKey,
            response_type: 'token',
            display: 'popup',
            scope: '',
            redirect_uri: redirect_uri,
            state: 'diauth_' + DI.guid()
        });

        if (opts.display === 'popup') {
            var win = window.open(DI.Auth.authorizeUrl + '?' + DI.QS.encode(opts), 'diauth', features);

            if (cb) {
                DI.Auth._active[opts.state] = {cb: cb, win: win};
                DI.Auth._popupMonitor();
            }
        } else {
            location.href = DI.Auth.authorizeUrl + '?' + DI.QS.encode(opts);
        }
    },

    logout: function(cb) {
        DI.api('/logout', cb);
        DI.Auth.setSession(null, 'notConnected');
    }
});

/**
 * Internal Authentication implementation.
 *
 * @class DI.Auth
 * @static
 * @access private
 */
DI.provide('Auth', {
    authorizeUrl: DI._domain['authorizeUrl'],
    _active: {},
    _receivedSession: null,

    /**
     * Check if session info are present in the URL fragment
     *
     * @access private
     */
    readFragment: function() {
        var h = window.location.hash.replace('%23', '#'), fragment = h.substr(h.lastIndexOf('#') + 1);
        if (fragment.indexOf('access_token=') >= 0 || fragment.indexOf('error=') >= 0) {
            var session = DI.QS.decode(fragment);

			if (window.name == 'diauth' && window.opener && window.opener.DI.Auth.setSession && window.opener.name != 'diauth') {
                // Display none helps prevent loading of some stuff
                document.documentElement.style.display = 'none';

                window.opener.DI.Auth.recvSession(session);
            } else if (session && ('state' in session) && session.state.indexOf('diauth_') == 0) { // Ensure it's our session
                // The session have been received as fragment, but we can't find a valid opener.
                // This happen either when the user is redirected to the authorization page or when the agent
                // doesn't fully support window.open, and replace the current window by the opened one
                // (i.e.: iPhone fullscreen webapp mode)
                if ('access_token' in session) {
                    DI.Auth._receivedSession = session;
                }
                // Remove the session from the fragment
                window.location.hash = h.substr(0, h.lastIndexOf('#'));
            } 
        }
    },

    /**
     * Recieve the authorization server response
     *
     * @access private
     * @param session {Object}  the new Session
     */
    recvSession: function(session) {
        if (!session) {
            DI.error('Received invalid session');
        }

        if ('error' in session) {
            DI.error('Received auth error `' + session.error + '\': ' + session.error_description);
        }

        if (!('state' in session)) {
            DI.error("Received a session with not `state' field");
            return;
        }

        if (!(session.state in DI.Auth._active)) {
            DI.error('Received a session from an inactive window');
            return;
        }

        DI.Auth._active[session.state].session = session;
    },

    /**
     * Set a new session value. Invokes all the registered subscribers
     * if needed.
     *
     * @access private
     * @param session {Object}  the new Session
     * @param status  {String}  the new status
     * @return        {Object}  the "response" object
     */
    setSession: function(session, status) {
        // detect special changes before changing the internal session
        var login = !DI._session && session,
            logout = DI._session && !session,
            both = false, // DI._session && session && DI._session.uid != session.uid,
            sessionChange = login || logout || (DI._session && session && DI._session.access_token != session.access_token),
            statusChange = status != DI._userStatus;

        if (session && 'expires_in' in session) {
            // CAVEAT: the expires here will actually only be valid on the client as end-user machines
            //         clock is rarely synced
            session.expires = Math.round(new Date().getTime() / 1000) + parseInt(session.expires_in, 10);
            delete session.expires_in;
        }

        var response = {
            session: session,
            status: status
        };

        DI._session = session;
        DI._userStatus = status;

        // If cookie support is enabled, set the cookie. Cookie support does not
        // rely on events, because we want the cookie to be set _before_ any of the
        // event handlers are fired. Note, this is a _weak_ dependency on Cookie.
        if (sessionChange && DI.Cookie && DI.Cookie.getEnabled()) {
            DI.Cookie.set(session);
        }

        // events
        if (statusChange) {
            /**
             * Fired when the status changes.
             *
             * @event auth.statusChange
             */
            DI.Event.fire('auth.statusChange', response);
        }
        if (logout || both) {
            /**
             * Fired when a logout action is performed.
             *
             * @event auth.logout
             */
            DI.Event.fire('auth.logout', response);
        }
        if (login || both) {
            /**
             * Fired when a login action is performed.
             *
             * @event auth.login
             */
            DI.Event.fire('auth.login', response);
        }
        if (sessionChange) {
            /**
             * Fired when the session changes. This includes a session being
             * refreshed, or a login or logout action.
             *
             * @event auth.sessionChange
             */
            DI.Event.fire('auth.sessionChange', response);
        }

        return response;
    },

    /**
     * Start and manage the window monitor interval. This allows us to invoke
     * the default callback for a window when the user closes the window
     * directly.
     *
     * @access private
     */
    _popupMonitor: function() {
        // check all open windows
        for (var id in DI.Auth._active) {
            if ('win' in DI.Auth._active[id]) {
                try {
                    // found a closed window
                    if (DI.Auth._active[id].win.closed) {
                        delete DI.Auth._active[id].win;
                        DI.Auth.recvSession({error:'access_denied', error_description:'Client closed the window', state:id});
                    }
                } catch (e) {
                }
            }

            if ('session' in DI.Auth._active[id]) {
                var callInfo = DI.Auth._active[id];
                delete DI.Auth._active[id];

                var session = callInfo.session;
                if ('access_token' in session) {
                    DI.Auth.setSession(session, 'connected');
                } else {
                    DI.Auth.setSession(null, 'notConnected');
                }

                if ('win' in callInfo) {
                    callInfo.win.close();
                }

                if ('cb' in callInfo) {
                    callInfo.cb({status: DI._userStatus, session: DI._session});
                }
            }
        }

        var hasActive = false;
        for (var id in DI.Auth._active) {
            hasActive = true;
            break;
        }
        if (hasActive && !DI.Auth._popupInterval) {
            // start the monitor if needed and it's not already running
            DI.Auth._popupInterval = window.setInterval(DI.Auth._popupMonitor, 100);
        } else if (!hasActive && DI.Auth._popupInterval) {
            // shutdown if we have nothing to monitor but it's running
            window.clearInterval(DI.Auth._popupInterval);
            DI.Auth._popupInterval = null;
        }
    }
});

DI.Auth.readFragment();
